#include "exi.h"
/* Define those to better describe your network interface. */
#define IFNAME0 'e'
#define IFNAME1 'n'


static void gcif_irq_handler(int channel, int event);
/*-----------------------------------------------------------------------------------*/

unsigned char eth_inb(int reg)
{
	unsigned long val;
	unsigned char res;
	val = (reg << 8) | 0x80000000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 4, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm(0, &res, 1, EXI_READ, 0);
	exi_sync(0);
	exi_deselect(0);
	return res;
}

unsigned char eth_exi_inb_slow(int reg)
{
	unsigned short val;
	unsigned char res;
	val = (reg << 8);
	exi_select(0, 2, 5);
	exi_imm(0, &val, 2, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm(0, &res, 1, EXI_READ, 0);
	exi_sync(0);
	exi_deselect(0);
	return res;
}

void eth_outb(int reg, unsigned char byte)
{
	unsigned long val;
	val = (reg << 8)|0xC0000000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 4, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm(0, &byte, 1, EXI_WRITE, 0);
	exi_sync(0);
	exi_deselect(0);
}

void eth_exi_outb(int reg, unsigned char byte)
{
	unsigned short val;
	val = (reg << 8)|0x4000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 2, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm(0, &byte, 1, EXI_WRITE, 0);
	exi_sync(0);
	exi_deselect(0);
}

unsigned char eth_exi_inb(int reg)
{
	unsigned short val;
	unsigned char res;
	val = reg << 8;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 2, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm(0, &res, 1, EXI_READ, 0);
	exi_sync(0);
	exi_deselect(0);
	return res;
}

void eth_exi_ins(int reg, void *res, int len)
{
	unsigned short val;
	val = reg << 8;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 2, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm_ex(0, res, len, EXI_READ);
	exi_sync(0);
	exi_deselect(0);
}

void eth_exi_outs(int reg, void *res, int len)
{
	unsigned short val;
	val = (reg << 8)|0x4000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 2, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm_ex(0, res, len, EXI_WRITE);
	exi_sync(0);
	exi_deselect(0);
}

void eth_ins(int reg, void *res, int len)
{
	unsigned long val;
	val = (reg << 8)|0x80000000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 4, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm_ex(0, res, len, EXI_READ);
	exi_sync(0);
	exi_deselect(0);
}

void eth_outs(int reg, void *res, int len)
{
	unsigned long val;
	val = (reg << 8)|0xC0000000;
	exi_select(0, 2, 5);
	exi_imm(0, &val, 4, EXI_WRITE, 0);
	exi_sync(0);
	exi_imm_ex(0, res, len, EXI_WRITE);
	exi_sync(0);
	exi_deselect(0);
}

static void inline gcif_service()
{
	int  p_read, p_write;
	int status = eth_inb(9) & eth_inb(8);
	if (!status)
	if (status & 2)
	{
		while (1)
		{
			p_write  = eth_inb(0x16);
			p_write |= eth_inb(0x17) << 8;
			p_read  = eth_inb(0x18);
			p_read |= eth_inb(0x19) << 8;

			if (p_write == p_read)
				break;
		}
		eth_outb(9, 2);
	}
	if (status & 8)
	{
		eth_outb(9, 8);
	}
	if (status & ~(8|2))
	{
		eth_outb(9, ~(8|2));
	}
}

void gcif_irq_handler(int channel, int event)
{
	int s;
	eth_exi_outb(2, 0);
	s = eth_exi_inb(3);
	if (s & 0x80)
	{
		eth_exi_outb(3, 0x80);
		eth_exi_outb(2, 0xF8);
		return;
	}
	if (s & 0x40)
	{
		eth_exi_outb(3, 0x40);
		eth_exi_outb(2, 0xF8);
		return;
	}
	if (s & 0x20)
	{
		eth_exi_outb(3, 0x20);
		eth_exi_outb(2, 0xF8);
		return;
	}
	if (s & 0x10)
	{
		eth_exi_outb(3, 0x10);
		eth_exi_outb(2, 0xF8);
		return;
	}
	if (s & 0x08)
	{
		eth_exi_outb(3, 0x08);
		eth_exi_outb(2, 0xF8);
		return;
	}
	eth_exi_outb(2, 0xF8);
}

static void
gc_init()
{
	eth_outb(0x60, 0);	// unknown
	eth_exi_inb_slow(0xF);
	eth_outb(0, 1);	 // reset
	eth_outb(0, 0);	 // /reset
	eth_exi_outs(4, "\xd1\x07\x75\x75", 2);
	eth_exi_outb(5, 0x4e);
	eth_outb(0x5b, eth_inb(0x5b)&~(1<<7));
	eth_outb(0x5e, 1);
	eth_outb(0x5c, eth_inb(0x5c)|4);
	eth_outb(1, 0x11);
//	eth_outb(1, 0);
	eth_outb(0x50, 0x80);
	// recvinit
	eth_outb(0xA,  0x1);
	eth_outb(0xB,  0x0);
	eth_outb(0x16, 0x1);
	eth_outb(0x17, 0x0);
	eth_outb(0x18, 0x1);
	eth_outb(0x19, 0x0);
	eth_outb(0x1a, 0xF);
	eth_outb(0x1b, 0);
	eth_outb(1, (eth_inb(1) & 0xFE) | 0x12);
	eth_outb(0, 8);
	eth_outb(0x32, 8);
	exi_request_irq(2, EXI_EVENT_IRQ, gcif_irq_handler);
	eth_exi_outb(0x2, 0xFF);
	eth_exi_outb(0x3, 0xFF);
 	eth_outb(8, 0xFF); // enable all IRQs
	eth_outb(9, 0xFF); // clear all irqs
}
